@@ -4,15 +4,16 @@
 #' series of temperature along with a daily date vector.
 #'
 #' @importFrom magrittr %>%
+#' @importFrom stats sd
 #' @importFrom plyr .
 #' @param data A data frame with three columns. They are headed \code{doy},
-#' which is the Julian day running from 1 to 366, but modified so that the
-#' day-of-year (doy) vector for non-leap-years runs 1...59 and then
-#' 61...366. For leap years the 60th day is February 29. The \code{date} column
-#' is a vector of dates of class \code{Date}, while \code{temp} is the
-#' temperature. Data of the appropriate format are created by the function
-#' \code{\link{make_whole}}, but your own data can be supplied if they meet the
-#' criteria specified by \code{\link{make_whole}}.
+#' \code{date} and \code{temp.} \code{doy} is the Julian day running from 1
+#' to 366, but modified so that the day-of-year (doy) vector for non-leap-years
+#' runs 1...59 and then 61...366. For leap years the 60th day is February 29.
+#' The \code{date} column is a vector of dates of class \code{Date}, while
+#' \code{temp} is the temperature. Data of the appropriate format are created
+#' by the function \code{\link{make_whole}}, but your own data can be supplied
+#' if they meet the criteria specified by \code{\link{make_whole}}.
 #' @param climatology_start The first full year from which the (varying by
 #' day-of-year) seasonal cycle and extremes threshold are calculated (full being
 #' 366 days if leap year, else 365 days). Note that a default value is provided
@@ -33,6 +34,8 @@
 #' @param smooth_percentile_width Boolean switch selecting whether to smooth the
 #' climatology and threshold percentile timeseries with a moving average of
 #' width \code{smooth_percentile}. Default is \code{TRUE}.
+#' @param clim_only Choose to calculate only the climatologies and not the
+#' events. Default is \code{FALSE}.
 #' @param min_duration Minimum duration for acceptance of detected MHWs.
 #' Default is \code{5} days.
 #' @param join_across_gaps Boolean switch indicating whether to join MHWs which
@@ -81,6 +84,8 @@
 #' deviations below the (100 - pctile)th percentile  (e.g., the 10th instead of
 #' 90th) for at least 5 days. Intensities are reported as negative values and
 #' represent the temperature anomaly below climatology.
+#' \item If only the climatology for the time series is required, and not the
+#' events themselves, this may be done by setting \code{clim_only} = TRUE.
 #' }
 #' The original Python algorithm was written by Eric Oliver, Institute for
 #' Marine and Antarctic Studies, University of Tasmania, Feb 2015, and is
@@ -101,11 +106,12 @@
 #'   \item{seas_clim_year}{Climatological seasonal cycle [deg. C].}
 #'   \item{thresh_clim_year}{Seasonally varying threshold (e.g., 90th
 #'   percentile) [deg. C].}
+#'   \item{var_clim_year}{Seasonally varying variance (standard deviation) [deg. C].}
 #'   \item{thresh_criterion}{Boolean indicating if \code{temp} exceeds
 #'   \code{thresh_clim_year}.}
 #'   \item{duration_criterion}{Boolean indicating whether periods of consecutive
 #'   \code{thresh_criterion} are >= \code{min_duration}.}
-#'   \item{event}{Boolean indicting if all criteria that define a MHW or MCS are
+#'   \item{event}{Boolean indicating if all criteria that define a MHW or MCS are
 #'   met.}
 #'   \item{event_no}{A sequential number indicating the ID and order of
 #'   occurence of the MHWs or MCSs.}
@@ -175,11 +181,14 @@ detect <-
            window_half_width = 5,
            smooth_percentile = TRUE,
            smooth_percentile_width = 31,
+           clim_only = FALSE,
            min_duration = 5,
            join_across_gaps = TRUE,
            max_gap = 2,
            max_pad_length = 3,
-           cold_spells = FALSE) {
+           cold_spells = FALSE
+           # verbose = TRUE, # to be implemented
+           ) {
 
     t_series <- data
     t_series$temp <- zoo::na.approx(t_series$temp, maxgap = max_pad_length)
@@ -191,13 +200,14 @@ detect <-
       stop("Bummer! Please provide a complete year for the end of the climatology.")
 
     clim_start <- paste(climatology_start, "01", "01", sep = "-")
-    if (t_series$date[1] > clim_start) {
-      stop(paste("The specified start date precedes the first day of series, which is", t_series$date[1]))
-    }
+    if (t_series$date[1] > clim_start)
+      stop(paste("The specified start date precedes the first day of series, which is",
+                 t_series$date[1]))
 
     clim_end <- paste(climatology_end, "12", "31", sep = "-")
     if (clim_end > t_series$date[nrow(t_series)])
-      stop(paste("The specified end date follows the last day of series, which is", t_series$date[nrow(t_series)]))
+      stop(paste("The specified end date follows the last day of series, which is",
+                 t_series$date[nrow(t_series)]))
 
     if (cold_spells)
       t_series$temp <- -t_series$temp
@@ -212,11 +222,14 @@ detect <-
     tDat[59:61, no_NA] <- zoo::na.approx(tDat[59:61, no_NA], maxgap = 1, na.rm = TRUE)
     tDat <- rbind(utils::tail(tDat, window_half_width),
                   tDat, utils::head(tDat, window_half_width))
-    seas_clim_year <- thresh_clim_year <- rep(NA, nrow(tDat))
+
+    seas_clim_year <- thresh_clim_year <- var_clim_year <- rep(NA, nrow(tDat))
 
     for (i in (window_half_width + 1):((nrow(tDat) - window_half_width))) {
       seas_clim_year[i] <-
-        mean(c(t(tDat[(i - (window_half_width)):(i + window_half_width), 2:ncol(tDat)])), na.rm = TRUE)
+        mean(
+          c(t(tDat[(i - (window_half_width)):(i + window_half_width), 2:ncol(tDat)])),
+          na.rm = TRUE)
       thresh_clim_year[i] <-
         raster::quantile(
           c(t(tDat[(i - (window_half_width)):(i + window_half_width), 2:ncol(tDat)])),
@@ -225,6 +238,11 @@ detect <-
           na.rm = TRUE,
           names = FALSE
         )
+      var_clim_year[i] <-
+        sd(
+          c(t(tDat[(i - (window_half_width)):(i + window_half_width), 2:ncol(tDat)])),
+          na.rm = TRUE
+        )
     }
 
     len_clim_year <- 366
@@ -232,7 +250,8 @@ detect <-
       data.frame(
         doy = tDat[(window_half_width + 1):((window_half_width) + len_clim_year), 1],
         seas_clim_year = seas_clim_year[(window_half_width + 1):((window_half_width) + len_clim_year)],
-        thresh_clim_year = thresh_clim_year[(window_half_width + 1):((window_half_width) + len_clim_year)]
+        thresh_clim_year = thresh_clim_year[(window_half_width + 1):((window_half_width) + len_clim_year)],
+        var_clim_year = var_clim_year[(window_half_width + 1):((window_half_width) + len_clim_year)]
       )
 
     if (smooth_percentile) {
@@ -256,218 +275,233 @@ detect <-
             circular = TRUE,
             na.rm = FALSE
           )
+        ) %>%
+        dplyr::mutate(
+          var_clim_year = raster::movingFun(
+            var_clim_year,
+            n = smooth_percentile_width,
+            fun = mean,
+            type = "around",
+            circular = TRUE,
+            na.rm = FALSE
+          )
         )
     }
 
-    t_series %<>% dplyr::inner_join(clim, by = "doy")
-    t_series$temp[is.na(t_series$temp)] <- t_series$seas_clim_year[is.na(t_series$temp)]
-
-    t_series$thresh_criterion <- t_series$temp > t_series$thresh_clim_year
-    ex1 <- rle(t_series$thresh_criterion)
-    ind1 <- rep(seq_along(ex1$lengths), ex1$lengths)
-    s1 <- split(zoo::index(t_series$thresh_criterion), ind1)
-    proto_events <- s1[ex1$values == TRUE]
-    index_stop <- index_start <- NULL ###
-    proto_events_rng <-
-      lapply(proto_events, function(x)
-        data.frame(index_start = min(x), index_stop = max(x)))
-
-    duration <- NULL ###
-
-    protoFunc <- function(proto_data) {
-      out <- proto_data %>%
-        dplyr::mutate(duration = index_stop - index_start + 1) %>%
-        dplyr::filter(duration >= min_duration) %>%
-        dplyr::mutate(date_start = t_series[index_start, "date"]) %>%
-        dplyr::mutate(date_stop = t_series[index_stop, "date"])
-    }
-
-    proto_events <- do.call(rbind, proto_events_rng) %>%
-      dplyr::mutate(event_no = cumsum(ex1$values[ex1$values == TRUE])) %>%
-      protoFunc()
-
-    t_series$duration_criterion <- rep(FALSE, nrow(t_series))
-
-    for (i in 1:nrow(proto_events)) {
-      t_series$duration_criterion[proto_events$index_start[i]:proto_events$index_stop[i]] <-
-        rep(TRUE, length = proto_events$duration[i])
-    }
-
-    ex2 <- rle(t_series$duration_criterion)
-    ind2 <- rep(seq_along(ex2$lengths), ex2$lengths)
-    s2 <- split(zoo::index(t_series$thresh_criterion), ind2)
-    proto_gaps <- s2[ex2$values == FALSE]
-    proto_gaps_rng <-
-      lapply(proto_gaps, function(x) data.frame(index_start = min(x), index_stop = max(x)))
-
-    proto_gaps <- do.call(rbind, proto_gaps_rng) %>%
-      dplyr::mutate(event_no = c(1:length(ex2$values[ex2$values == FALSE]))) %>%
-      dplyr::mutate(duration = index_stop - index_start + 1)
-
-    if (any(proto_gaps$duration >= 1 & proto_gaps$duration <= max_gap)) {
-      proto_gaps %<>%
-        dplyr::mutate(date_start = t_series[index_start, "date"]) %>%
-        dplyr::mutate(date_stop = t_series[index_stop, "date"]) %>%
-        dplyr::filter(duration >= 1 & duration <= max_gap)
+    if(clim_only) {
+      t_series <- merge(data, clim, by = "doy")
+      t_series <- t_series[order(t_series$date),]
+      return(t_series)
     } else {
-      join_across_gaps <- FALSE
-    }
+      t_series %<>% dplyr::inner_join(clim, by = "doy")
+      t_series$temp[is.na(t_series$temp)] <- t_series$seas_clim_year[is.na(t_series$temp)]
+      t_series$thresh_criterion <- t_series$temp > t_series$thresh_clim_year
+      ex1 <- rle(t_series$thresh_criterion)
+      ind1 <- rep(seq_along(ex1$lengths), ex1$lengths)
+      s1 <- split(zoo::index(t_series$thresh_criterion), ind1)
+      proto_events <- s1[ex1$values == TRUE]
+      index_stop <- index_start <- NULL ###
+      proto_events_rng <-
+        lapply(proto_events, function(x)
+          data.frame(index_start = min(x), index_stop = max(x)))
 
-    if (join_across_gaps) {
-      t_series$event <- t_series$duration_criterion
-      for (i in 1:nrow(proto_gaps)) {
-        t_series$event[proto_gaps$index_start[i]:proto_gaps$index_stop[i]] <-
-          rep(TRUE, length = proto_gaps$duration[i])
+      duration <- NULL ###
+
+      protoFunc <- function(proto_data) {
+        out <- proto_data %>%
+          dplyr::mutate(duration = index_stop - index_start + 1) %>%
+          dplyr::filter(duration >= min_duration) %>%
+          dplyr::mutate(date_start = t_series$date[index_start]) %>%
+          dplyr::mutate(date_stop = t_series$date[index_stop])
       }
-    } else {
-      t_series$event <- t_series$duration_criterion
-    }
 
-    ex3 <- rle(t_series$event)
-    ind3 <- rep(seq_along(ex3$lengths), ex3$lengths)
-    s3 <- split(zoo::index(t_series$event), ind3)
-    events <- s3[ex3$values == TRUE]
-    event_no <- NULL ###
-    events_rng <-
-      lapply(events, function(x)
-        data.frame(index_start = min(x), index_stop = max(x)))
+      proto_events <- do.call(rbind, proto_events_rng) %>%
+        dplyr::mutate(event_no = cumsum(ex1$values[ex1$values == TRUE])) %>%
+        protoFunc()
 
-    events <- do.call(rbind, events_rng) %>%
-      dplyr::mutate(event_no = cumsum(ex3$values[ex3$values == TRUE])) %>%
-      protoFunc()
+      t_series$duration_criterion <- rep(FALSE, nrow(t_series))
 
-    t_series$event_no <- rep(NA, nrow(t_series))
-    for (i in 1:nrow(events)) {
-      t_series$event_no[events$index_start[i]:events$index_stop[i]] <-
-        rep(i, length = events$duration[i])
-    }
+      for (i in 1:nrow(proto_events)) {
+        t_series$duration_criterion[proto_events$index_start[i]:proto_events$index_stop[i]] <-
+          rep(TRUE, length = proto_events$duration[i])
+      }
 
-    events_list <- plyr::dlply(events, .(event_no), function(x)
-      with(
-        t_series,
-        data.frame(
-          date = c(date[x$index_start:x$index_stop]),
-          temp = c(temp[x$index_start:x$index_stop]),
-          seas_clim_year = c(seas_clim_year[x$index_start:x$index_stop]),
-          thresh_clim_year = c(thresh_clim_year[x$index_start:x$index_stop]),
-          mhw_rel_seas = c(temp[x$index_start:x$index_stop]) - c(seas_clim_year[x$index_start:x$index_stop]),
-          mhw_rel_thresh = c(temp[x$index_start:x$index_stop]) - c(thresh_clim_year[x$index_start:x$index_stop]),
-          rel_thresh_norm = c(temp[x$index_start:x$index_stop]) - c(thresh_clim_year[x$index_start:x$index_stop]) /
-            c(thresh_clim_year[x$index_start:x$index_stop]) - c(seas_clim_year[x$index_start:x$index_stop])
+      ex2 <- rle(t_series$duration_criterion)
+      ind2 <- rep(seq_along(ex2$lengths), ex2$lengths)
+      s2 <- split(zoo::index(t_series$thresh_criterion), ind2)
+      proto_gaps <- s2[ex2$values == FALSE]
+      proto_gaps_rng <-
+        lapply(proto_gaps, function(x) data.frame(index_start = min(x), index_stop = max(x)))
+
+      proto_gaps <- do.call(rbind, proto_gaps_rng) %>%
+        dplyr::mutate(event_no = c(1:length(ex2$values[ex2$values == FALSE]))) %>%
+        dplyr::mutate(duration = index_stop - index_start + 1)
+
+      if (any(proto_gaps$duration >= 1 & proto_gaps$duration <= max_gap)) {
+        proto_gaps %<>%
+          dplyr::mutate(date_start = t_series$date[index_start]) %>%
+          dplyr::mutate(date_stop = t_series$date[index_stop]) %>%
+          dplyr::filter(duration >= 1 & duration <= max_gap)
+      } else {
+        join_across_gaps <- FALSE
+      }
+
+      if (join_across_gaps) {
+        t_series$event <- t_series$duration_criterion
+        for (i in 1:nrow(proto_gaps)) {
+          t_series$event[proto_gaps$index_start[i]:proto_gaps$index_stop[i]] <-
+            rep(TRUE, length = proto_gaps$duration[i])
+        }
+      } else {
+        t_series$event <- t_series$duration_criterion
+      }
+
+      ex3 <- rle(t_series$event)
+      ind3 <- rep(seq_along(ex3$lengths), ex3$lengths)
+      s3 <- split(zoo::index(t_series$event), ind3)
+      events <- s3[ex3$values == TRUE]
+      event_no <- NULL ###
+      events_rng <-
+        lapply(events, function(x)
+          data.frame(index_start = min(x), index_stop = max(x)))
+
+      events <- do.call(rbind, events_rng) %>%
+        dplyr::mutate(event_no = cumsum(ex3$values[ex3$values == TRUE])) %>%
+        protoFunc()
+
+      t_series$event_no <- rep(NA, nrow(t_series))
+      for (i in 1:nrow(events)) {
+        t_series$event_no[events$index_start[i]:events$index_stop[i]] <-
+          rep(i, length = events$duration[i])
+      }
+
+      events_list <- plyr::dlply(events, .(event_no), function(x)
+        with(
+          t_series,
+          data.frame(
+            date = c(date[x$index_start:x$index_stop]),
+            temp = c(temp[x$index_start:x$index_stop]),
+            seas_clim_year = c(seas_clim_year[x$index_start:x$index_stop]),
+            thresh_clim_year = c(thresh_clim_year[x$index_start:x$index_stop]),
+            mhw_rel_seas = c(temp[x$index_start:x$index_stop]) - c(seas_clim_year[x$index_start:x$index_stop]),
+            mhw_rel_thresh = c(temp[x$index_start:x$index_stop]) - c(thresh_clim_year[x$index_start:x$index_stop]),
+            rel_thresh_norm = c(temp[x$index_start:x$index_stop]) - c(thresh_clim_year[x$index_start:x$index_stop]) /
+              c(thresh_clim_year[x$index_start:x$index_stop]) - c(seas_clim_year[x$index_start:x$index_stop])
+          )
+        ))
+
+      int_mean <- int_max <- int_cum <- int_mean_rel_thresh <-
+        int_max_rel_thresh <- int_cum_rel_thresh <- int_mean_abs <-
+        int_max_abs <- int_cum_abs <- int_mean_norm <- int_max_norm <-
+        temp <- doy <- NULL ###
+      events$date_peak <-
+        plyr::ldply(events_list, function(x)
+          x$date[x$mhw_rel_seas == max(x$mhw_rel_seas)][1])[, 2]
+      events$int_mean <-
+        plyr::ldply(events_list, function(x) mean(x$mhw_rel_seas))[, 2]
+      events$int_max <-
+        plyr::ldply(events_list, function(x) max(x$mhw_rel_seas))[, 2]
+      events$int_var <-
+        plyr::ldply(events_list, function(x) sqrt(stats::var(x$mhw_rel_seas)))[, 2]
+      events$int_cum <-
+        plyr::ldply(events_list, function(x) max(cumsum(x$mhw_rel_seas)))[, 2]
+      events$int_mean_rel_thresh <-
+        plyr::ldply(events_list, function(x) mean(x$mhw_rel_thresh))[, 2]
+      events$int_max_rel_thresh <-
+        plyr::ldply(events_list, function(x) max(x$mhw_rel_thresh))[, 2]
+      events$int_var_rel_thresh <-
+        plyr::ldply(events_list, function(x) sqrt(stats::var(x$mhw_rel_thresh)))[, 2]
+      events$int_cum_rel_thresh <-
+        plyr::ldply(events_list, function(x) max(cumsum(x$mhw_rel_thresh)))[, 2]
+      events$int_mean_abs <-
+        plyr::ldply(events_list, function(x) mean(x$temp))[, 2]
+      events$int_max_abs <-
+        plyr::ldply(events_list, function(x) max(x$temp))[, 2]
+      events$int_var_abs <-
+        plyr::ldply(events_list, function(x) sqrt(stats::var(x$temp)))[, 2]
+      events$int_cum_abs <-
+        plyr::ldply(events_list, function(x) max(cumsum(x$temp)))[, 2]
+      events$int_mean_norm <-
+        plyr::ldply(events_list, function(x) mean(x$rel_thresh_norm))[, 2]
+      events$int_max_norm <-
+        plyr::ldply(events_list, function(x) max(x$rel_thresh_norm))[, 2]
+
+      mhw_rel_seas <- t_series$temp - t_series$seas_clim_year
+      A <- mhw_rel_seas[events$index_start]
+      B <- t_series$temp[events$index_start - 1]
+      C <- t_series$seas_clim_year[events$index_start - 1]
+      mhw_rel_seas_start <- 0.5 * (A + B - C)
+      start_type <- ifelse(
+        events$index_start > 1,
+        "case1",
+        ifelse(
+          events$index_start == 1 &
+            difftime(events$date_peak, events$date_start, units = "days") > 0,
+          "case2",
+          "case3"
         )
-      ))
+      )[1]
+      rateOnset <- function(x, type) {
+        switch(
+          type,
+          case1 = (x$int_max - mhw_rel_seas_start) / (as.numeric(
+            difftime(x$date_peak, x$date_start, units = "days")
+          ) + 0.5),
+          case2 = (x$int_max - A) / 1,
+          case3 = (x$int_max - A) / as.numeric(difftime(x$date_peak, x$date_start, units = "days"))
+        )
+      }
+      events$rate_onset <- rateOnset(events, start_type)
 
-    int_mean <- int_max <- int_cum <- int_mean_rel_thresh <-
-      int_max_rel_thresh <- int_cum_rel_thresh <- int_mean_abs <-
-      int_max_abs <- int_cum_abs <- int_mean_norm <- int_max_norm <-
-      temp <- doy <- NULL ###
-    events$date_peak <-
-      plyr::ldply(events_list, function(x)
-        x$date[x$mhw_rel_seas == max(x$mhw_rel_seas)][1])[, 2]
-    events$int_mean <-
-      plyr::ldply(events_list, function(x) mean(x$mhw_rel_seas))[, 2]
-    events$int_max <-
-      plyr::ldply(events_list, function(x) max(x$mhw_rel_seas))[, 2]
-    events$int_var <-
-      plyr::ldply(events_list, function(x) sqrt(stats::var(x$mhw_rel_seas)))[, 2]
-    events$int_cum <-
-      plyr::ldply(events_list, function(x) max(cumsum(x$mhw_rel_seas)))[, 2]
-    events$int_mean_rel_thresh <-
-      plyr::ldply(events_list, function(x) mean(x$mhw_rel_thresh))[, 2]
-    events$int_max_rel_thresh <-
-      plyr::ldply(events_list, function(x) max(x$mhw_rel_thresh))[, 2]
-    events$int_var_rel_thresh <-
-      plyr::ldply(events_list, function(x) sqrt(stats::var(x$mhw_rel_thresh)))[, 2]
-    events$int_cum_rel_thresh <-
-      plyr::ldply(events_list, function(x) max(cumsum(x$mhw_rel_thresh)))[, 2]
-    events$int_mean_abs <-
-      plyr::ldply(events_list, function(x) mean(x$temp))[, 2]
-    events$int_max_abs <-
-      plyr::ldply(events_list, function(x) max(x$temp))[, 2]
-    events$int_var_abs <-
-      plyr::ldply(events_list, function(x) sqrt(stats::var(x$temp)))[, 2]
-    events$int_cum_abs <-
-      plyr::ldply(events_list, function(x) max(cumsum(x$temp)))[, 2]
-    events$int_mean_norm <-
-      plyr::ldply(events_list, function(x) mean(x$rel_thresh_norm))[, 2]
-    events$int_max_norm <-
-      plyr::ldply(events_list, function(x) max(x$rel_thresh_norm))[, 2]
+      D <- mhw_rel_seas[events$index_stop]
+      E <- t_series$temp[events$index_stop + 1]
+      F <- t_series$seas_clim_year[events$index_stop + 1]
+      mhw_rel_seas_end <- 0.5 * (D + E - F)
+      stop_type <- ifelse(
+        events$index_stop < nrow(t_series),
+        "case4",
+        ifelse(
+          events$index_stop == nrow(t_series) &
+            difftime(events$date_peak, t_series[nrow(t_series), "date"], units = "days") < 0,
+          "case5",
+          "case6"
+        )
+      )[nrow(events)]
 
-    mhw_rel_seas <- t_series$temp - t_series$seas_clim_year
-    A <- mhw_rel_seas[events$index_start]
-    B <- t_series$temp[events$index_start - 1]
-    C <- t_series$seas_clim_year[events$index_start - 1]
-    mhw_rel_seas_start <- 0.5 * (A + B - C)
-    start_type <- ifelse(
-      events$index_start > 1,
-      "case1",
-      ifelse(
-        events$index_start == 1 &
-          difftime(events$date_peak, events$date_start, units = "days") > 0,
-        "case2",
-        "case3"
-      )
-    )[1]
-    rateOnset <- function(x, type) {
-      switch(
-        type,
-        case1 = (x$int_max - mhw_rel_seas_start) / (as.numeric(
-          difftime(x$date_peak, x$date_start, units = "days")
-        ) + 0.5),
-        case2 = (x$int_max - A) / 1,
-        case3 = (x$int_max - A) / as.numeric(difftime(x$date_peak, x$date_start, units = "days"))
-      )
+      rateDecline <- function(x, type) {
+        switch(
+          type,
+          case4 = (x$int_max - mhw_rel_seas_end) / (as.numeric(
+            difftime(x$date_stop, x$date_peak, units = "days")
+          ) + 0.5),
+          case5 = (x$int_max - mhw_rel_seas_end) / 1,
+          case6 = (x$int_max - mhw_rel_seas_end) / as.numeric(difftime(x$date_stop, x$date_peak, units = "days"))
+        )
+      }
+      events$rate_decline <- rateDecline(events, stop_type)
+
+      if (cold_spells) {
+        events <- events %>% dplyr::mutate(
+          int_mean = -int_mean,
+          int_max = -int_max,
+          int_cum = -int_cum,
+          int_mean_rel_thresh = -int_mean_rel_thresh,
+          int_max_rel_thresh = -int_max_rel_thresh,
+          int_cum_rel_thresh = -int_cum_rel_thresh,
+          int_mean_abs = -int_mean_abs,
+          int_max_abs = -int_max_abs,
+          int_cum_abs = -int_cum_abs,
+          int_mean_norm = -int_mean_norm,
+          int_max_norm = -int_max_norm
+        )
+        t_series <- t_series %>% dplyr::mutate(
+          temp = -temp,
+          seas_clim_year = -seas_clim_year,
+          thresh_clim_year = -thresh_clim_year
+        )
+      }
+
+      list(clim = dplyr::group_by(t_series, event_no),
+           event = dplyr::group_by(events, event_no))
     }
-    events$rate_onset <- rateOnset(events, start_type)
-
-    D <- mhw_rel_seas[events$index_stop]
-    E <- t_series$temp[events$index_stop + 1]
-    F <- t_series$seas_clim_year[events$index_stop + 1]
-    mhw_rel_seas_end <- 0.5 * (D + E - F)
-    stop_type <- ifelse(
-      events$index_stop < nrow(t_series),
-      "case4",
-      ifelse(
-        events$index_stop == nrow(t_series) &
-          difftime(events$date_peak, t_series[nrow(t_series), "date"], units = "days") < 0,
-        "case5",
-        "case6"
-      )
-    )[nrow(events)]
-
-    rateDecline <- function(x, type) {
-      switch(
-        type,
-        case4 = (x$int_max - mhw_rel_seas_end) / (as.numeric(
-          difftime(x$date_stop, x$date_peak, units = "days")
-        ) + 0.5),
-        case5 = (x$int_max - mhw_rel_seas_end) / 1,
-        case6 = (x$int_max - mhw_rel_seas_end) / as.numeric(difftime(x$date_stop, x$date_peak, units = "days"))
-      )
-    }
-    events$rate_decline <- rateDecline(events, stop_type)
-
-    if (cold_spells) {
-      events <- events %>% dplyr::mutate(
-        int_mean = -int_mean,
-        int_max = -int_max,
-        int_cum = -int_cum,
-        int_mean_rel_thresh = -int_mean_rel_thresh,
-        int_max_rel_thresh = -int_max_rel_thresh,
-        int_cum_rel_thresh = -int_cum_rel_thresh,
-        int_mean_abs = -int_mean_abs,
-        int_max_abs = -int_max_abs,
-        int_cum_abs = -int_cum_abs,
-        int_mean_norm = -int_mean_norm,
-        int_max_norm = -int_max_norm
-      )
-      t_series <- t_series %>% dplyr::mutate(
-        temp = -temp,
-        seas_clim_year = -seas_clim_year,
-        thresh_clim_year = -thresh_clim_year
-      )
-    }
-
-    list(clim = dplyr::group_by(t_series, event_no),
-         event = dplyr::group_by(events, event_no))
   }
